@@ -19,19 +19,7 @@
 
 import os, sys, shutil, subprocess, collections, time, datetime, shlex, getopt, stat
 from .gitignore_parser import parse_gitignore
-
-# from python3.3 tree: Lib/shlex.py (shlex.quote not in python3.2)
-import re
-_find_unsafe = re.compile(r'[^\w@%+=:,./-]', re.ASCII).search
-def quote(s):
-    """Return a shell-escaped version of the string *s*."""
-    if not s:
-        return "''"
-    if _find_unsafe(s) is None:
-        return s
-    # use single quotes, and put single quotes into double quotes
-    # the string $'b is then quoted as '$'"'"'b'
-    return "'" + s.replace("'", "'\"'\"'") + "'"
+from .utils import get_config_path, readlines, quote
 
 def quotepath(path):
 	return b"'" + path.replace(b"'", b"'\"'\"'") + b"'"
@@ -373,6 +361,14 @@ def read_file_record(gen):
 		else:
 			sys.exit("Error: snap filelists not coherent.")
 
+	print("=========")
+	print(f"i={i}")
+	print(f"p={p}")
+	print(f"t={t}")
+	print(f"d={d}")
+	print(f"s={s}")
+	print(f"perms={perms}")
+
 	d = d.split(".")[0]	# truncate date to seconds
 	if t=="d": d=s="0"	# ignore dates/size for dirs (set to zero)
 	if ignoreperms: perms = ""
@@ -393,7 +389,8 @@ def load_orig(ssh1,dir1name, ssh2,dir2name):
 
 	ignores1 = get_ignores(ignorefile1, ssh1,dir1name)
 	ignores2 = get_ignores(ignorefile2, ssh2,dir2name)
-	ignores = parse_gitignore(ignores1 | ignores2)
+	ignores_global = readlines(get_config_path("ignore"))
+	ignores = parse_gitignore(ignores1 | ignores2 | set(ignores_global))
 
 	common_snaps = snaps1.intersection(snaps2)
 	orig = collections.OrderedDict()
