@@ -3,8 +3,11 @@ import sys
 import json
 import subprocess
 import shlex
+import shutil
 from coleo import Argument, auto_cli, default
 
+from .utils import get_config, get_config_path, write_config, readlines, writelines, quote
+
 
 def q(message=None):
     if message:
@@ -21,31 +24,6 @@ def _check_remote(cfg, name, msg=None):
     return cfg[name]
 
 
-def _get_config_path(name=None):
-    path = os.path.expanduser("~/.config/synecure")
-    if name is not None:
-        path = os.path.join(path, name)
-    return path
-
-
-def _get_config(name):
-    path = _get_config_path(name)
-    if not os.path.exists(path):
-        return {}
-    with open(path) as f:
-        return json.load(f)
-
-
-def _write_config(name, cfg, silent=False):
-    path = _get_config_path(name)
-    cdir = os.path.dirname(path)
-    os.makedirs(cdir, exist_ok=True)
-    with open(path, "w") as f:
-        json.dump(cfg, f, indent=4)
-    if not silent:
-        print(f"Written config at: {path}")
-
-
 def _sort_paths(remote):
     return sorted(remote["paths"].items(), key=lambda kv: -len(kv[0]))
 
@@ -63,11 +41,11 @@ def entry_sy():
     auto_cli(main)
 
 
-def entry_sy_remote():
+def entry_sy_config():
     commands = {}
     for name, value in globals().items():
         parts = name.split("_")
-        if parts[0] == "remote" and len(parts) > 1:
+        if parts[0] == "config" and len(parts) > 1:
             curr = commands
             for part in parts[1:-1]:
                 curr = curr.setdefault(part, {})
@@ -108,8 +86,11 @@ def main():
     # [alias: -i]
     interactive: Argument & bool = default(False)
 
-    remotes = _get_config("remotes.json")
-    directories = _get_config("directories.json")
+    # Use SSHFS to perform the update
+    sshfs: Argument & bool = default(False)
+
+    remotes = get_config("remotes.json")
+    directories = get_config("directories.json")
 
     if list:
         for path, dest in directories.items():
@@ -124,6 +105,7 @@ def main():
         filename = os.path.realpath(os.path.expanduser(filename))
         commands += plan_sync(
             filename, remote, remotes, directories,
+            sshfs=sshfs,
             dry=dry_run,
             verbose=verbose,
             interactive=interactive
@@ -138,23 +120,35 @@ def main():
         if not show_plan:
             subprocess.run(command)
 
-    _write_config("directories.json", directories, silent=True)
+    write_config("directories.json", directories, silent=True)
+
+
+def _check_dir(url, path, port):
+    if url:
+        args = ["ssh", url]
+        if port:
+            args.extend(["-p", "22005"])
+        args.append(f"test -d {quote(path)}")
+        result = subprocess.call(args)
+        return result == 0
+
+    else:
+        return os.path.isdir(path)
 
 
 def plan_sync(path, remote_name, remotes, directories,
-              dry=False, verbose=False, interactive=False):
-    commands = []
-
+              dry=False, verbose=False, interactive=False,
+              sshfs=False):
     if remote_name is None:
         if path not in directories:
             q("Please specify a destination")
         regdest = directories[path]
         assert regdest is not None
-        commands += plan_sync(path, regdest, remotes, directories,
+        return plan_sync(path, regdest, remotes, directories,
             dry=dry,
-            interactive=interactive
+            interactive=interactive,
+            sshfs=sshfs
         )
-        return commands
 
     directories[path] = remote_name
 
@@ -177,7 +171,36 @@ def plan_sync(path, remote_name, remotes, directories,
     else:
         print(f"# Unknown remote type: {remote['type']}")
 
-    if os.path.isdir(path):
+    if sshfs and remote["type"] == "ssh":
+        # Mount in a standard place in .config to ease debugging
+        mountpoint = get_config_path("mnt")
+        os.makedirs(mountpoint, exist_ok=True)
+        commands = [
+            ["ssh", remote["url"], "-p", str(remote["port"]), f"mkdir -p {destpath}"],
+            ["sshfs", "-o", "Compression=no", dest, "-p", str(remote["port"]), mountpoint]
+        ]
+        print(f"# MOUNT REMOTE    {mountpoint}")
+        commands += _plan_commands(remote, path, mountpoint, mountpoint, dry=dry,
+            interactive=interactive)
+        umount = shutil.which("fusermount") or "umount"
+        commands.append([umount, mountpoint])
+        return commands
+
+    else:
+        return _plan_commands(remote, path, dest, destpath, dry=dry,
+            interactive=interactive)
+
+
+def _plan_commands(remote, path, dest, destpath, dry, interactive):
+
+    commands = []
+
+    if os.path.exists(path):
+        isdir = os.path.isdir(path)
+    else:
+        isdir = _check_dir(remote["url"], destpath, remote["port"])
+
+    if isdir:
         # Use bsync to synchronize both directories
 
         cmdopts = ["-d"]
@@ -237,7 +260,7 @@ def remote_add():
     # [alias: -p]
     port: Argument = default(22)
 
-    cfg = _get_config("remotes.json")
+    cfg = get_config("remotes.json")
     if "@" in url:
         cfg[name] = {
             "type": "ssh",
@@ -256,23 +279,23 @@ def remote_add():
                 os.getenv("HOME"): os.path.realpath(os.path.expanduser(url))
             }
         }
-    _write_config("remotes.json", cfg)
+    write_config("remotes.json", cfg)
 
 
-def remote_view():
+def config_view():
     # Name of the remote
     # [positional: ?]
     name: Argument
 
-    cfg = _get_config("remotes.json")
+    cfg = get_config("remotes.json")
     if name is not None:
         cfg = _check_remote(cfg, name)
 
     print(json.dumps(cfg, indent=4))
 
 
-def remote_list():
-    cfg = _get_config("remotes.json")
+def config_list():
+    cfg = get_config("remotes.json")
     for name, defn in cfg.items():
         if defn['port'] in (None, 22):
             port = ""
@@ -283,7 +306,7 @@ def remote_list():
             print(f"    {local_path:30} -> :{remote_path}")
 
 
-def remote_path():
+def config_path():
     # Name of the remote
     # [positional]
     name: Argument
@@ -304,7 +327,7 @@ def remote_path():
     # [alias: -r]
     remove: Argument = default(None)
 
-    cfg = _get_config("remotes.json")
+    cfg = get_config("remotes.json")
     remote = _check_remote(cfg, name, msg="Nothing to do")
 
     if list:
@@ -315,7 +338,7 @@ def remote_path():
         if remove not in remote["paths"]:
             q(f"Source path '{remove}' is not mapped")
         del remote["paths"][remove]
-        _write_config("remotes.json", cfg)
+        write_config("remotes.json", cfg)
 
     else:
         if source is None:
@@ -324,15 +347,52 @@ def remote_path():
             q("DEST must be specified")
         paths = remote["paths"]
         paths[source] = dest
-        _write_config("remotes.json", cfg)
+        write_config("remotes.json", cfg)
 
 
-def remote_remove():
+def config_remove():
     # Name of the remote
     # [positional]
     name: Argument
 
-    cfg = _get_config("remotes.json")
+    cfg = get_config("remotes.json")
     _check_remote(cfg, name, msg="Nothing to remove")
     del cfg[name]
-    _write_config("remotes.json", cfg)
+    write_config("remotes.json", cfg)
+
+
+def config_ignore():
+    # Patterns to ignore
+    # [positional: *]
+    patterns: Argument = default([])
+
+    ign = get_config_path("ignore")
+    lines = readlines(ign)
+
+    for line in lines:
+        print(f" {line}")
+    for pattern in patterns:
+        if pattern not in lines:
+            print(f"+{pattern}")
+            lines.append(pattern)
+
+    writelines(ign, lines)
+
+
+def config_unignore():
+    # Patterns to ignore
+    # [positional: *]
+    patterns: Argument = default([])
+
+    ign = get_config_path("ignore")
+    lines = readlines(ign)
+
+    new_lines = []
+    for line in lines:
+        if line in patterns:
+            print(f"-{line}")
+        else:
+            print(f" {line}")
+            new_lines.append(line)
+
+    writelines(ign, new_lines)
